#!/usr/bin/env python

import distutils.dir_util
import grp
import json
import os
import pwd
import re
import shutil
import subprocess
import sys

import charmhelpers.core.hookenv
import charmhelpers.fetch
import charmhelpers.contrib.charmsupport.nrpe

from charmhelpers.core.hookenv import (
    config as config_get,
    relation_set,
    relation_get,
    relation_ids,
)

hooks = charmhelpers.core.hookenv.Hooks()
log = charmhelpers.core.hookenv.log


###############################################################################
# Global variables
###############################################################################
site_name = "ci-airlines.canonical.com"
site_path = "/srv/{}".format(site_name)
log_path = "{}/logs".format(site_path)
site_conf_path = "/etc/apache2/sites-available/{}".format(site_name)

config_data = config_get()
ticket_manager_url = config_data.get('ticket_manager_url', '')

site_conf = """
<VirtualHost *:80>

    #ServerName {site_name}

    CustomLog {log_path}/access.log combined
    ErrorLog {log_path}/error.log

    DocumentRoot {site_path}

    <Directory />
         Options FollowSymLinks
         AllowOverride None
    </Directory>

    <Directory {site_path}>
         Options Indexes FollowSymLinks MultiViews
         AllowOverride None
         Order allow,deny
         Allow from all
    </Directory>

    RewriteEngine       on
    RewriteRule         ^/api/(.*)      {ticket_manager_url}/$1  [P]

</VirtualHost>
""".format(
    site_name=site_name,
    site_path=site_path,
    log_path=log_path,
    ticket_manager_url=ticket_manager_url,
)

web_uid = pwd.getpwnam('www-data')[2]
web_gid = grp.getgrnam('www-data')[2]


###############################################################################
# Supporting functions
###############################################################################

def _log(msg):
    log("JAT: {msg}".format(msg=msg))

def _setup_env():

    if os.path.exists(site_path):
        shutil.rmtree(site_path)

    os.makedirs(site_path)

    if not os.path.exists(log_path):
        os.makedirs(log_path)

    # XXX: make these config options.
    os.chown(site_path, web_uid, web_gid)

    distutils.dir_util.copy_tree('files/webroot/', site_path)
    for root, dirs, files in os.walk(site_path):
        map(
            lambda file: os.chown(
                os.path.join(root, file), web_uid, web_gid
            ),
            files
        )
        map(
            lambda dir: os.chown(
                os.path.join(root, dir), web_uid, web_gid
            ),
            dirs
        )

    regex = re.compile(ur'^test.*\.html')
    for f in os.listdir(site_path):
        if regex.match(f):
            os.remove(os.path.join(site_path, f))

    with open(os.path.join(site_path, 'config.json'), 'w') as fp:
        config_data = config_get()
        json.dump(config_data, fp)


@hooks.hook()
def install():
    _setup_env()

    with open(site_conf_path, 'w') as fp:
        fp.write(site_conf)


def ensite():
    subprocess.call(['a2ensite', site_name])
    subprocess.call(['a2dissite', "default"])


def dissite():
    subprocess.call(['a2ensite', "default"])
    subprocess.call(['a2dissite', site_name])

@hooks.hook()
def website_relation_changed():
    _log("changed relationship")  # assumes there's only one for now.
    relation_data = relation_get()
    _log("relation_data: {}".format(relation_data))

@hooks.hook()
def website_relation_joined():
    _log("joined relationship")  # assumes there's only one for now.
    relation_data = relation_get()
    _log("relation_data: {}".format(relation_data))
    ensite()


@hooks.hook()
def website_relation_broken():
    config_data = config_get()
    _log("config_data: {}".format(config_data))
    _log("departing relationship")  # assumes there's only one for now.
    dissite()

@hooks.hook()
def stop():
    _log("stopping webui")


@hooks.hook()
def start():
    _log("starting webui")


@hooks.hook()
def config_changed():
    pass

@hooks.hook()
def upgrade_charm():
    install()


if __name__ == "__main__":
    hooks.execute(sys.argv)
