# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import yaml
import json
import urllib2

HEADERS = {'content-type': 'application/json'}
API_URL = '/api/v1/'
TICKET_BASE = API_URL + 'ticket/'
SPU_BASE = API_URL + 'spu/'
SOURCEPACKAGE_BASE = API_URL + 'sourcepackage/'
ARTIFACT_BASE = API_URL + 'artifact/'
SUBTICKET_BASE = API_URL + 'subticket/'

config_file = os.path.join(os.environ["HOME"], '.ubuntu-ci/config.yaml')
try:
    with open(config_file) as fp:
        config = yaml.safe_load(fp)
except IOError, e:
    sys.exit("Missing config info. " +
             "Add 'ci_url' to {}".format(config_file))

CI_URL = config['ci_url']


def post(url, data, headers=HEADERS, patch=False):
    req = urllib2.Request(url=url, data=json.dumps(data), headers=HEADERS)
    f = urllib2.urlopen(req)
    return f.info().dict['location']


def patch(url, data, headers=HEADERS):
    req = urllib2.Request(url=url, data=json.dumps(data), headers=HEADERS)
    req.get_method = lambda: 'PATCH'
    urllib2.urlopen(req)


def get(url):
    f = urllib2.urlopen(url)

    json_data = f.read()
    return json.loads(json_data)


def parse_id(location):
    try:
        id = location.split('/')[6]
        int(id)
    except ValueError, exc:
        raise exc
    return id


def get_sourcepackage_uri(sourcepackage):
    url = CI_URL + SOURCEPACKAGE_BASE + '?name__iexact=' + sourcepackage
    f = urllib2.urlopen(url)
    json_data = f.read()
    data = json.loads(json_data)
    # For this to work it's required that the source package is already added,
    # otherwise it fails with index error because set returns empty.
    return data['objects'][0]['resource_uri']
