#!/usr/bin/env python
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import mock
import imp

from unittest import TestCase

from ci_libs.ticket import SubTicket, Ticket


class SubTicketTestCase(TestCase):

    def setUp(self):
        self.changes = os.path.join(os.path.dirname(__file__),
                                    'data/foobar_0.1-1_source.changes')

    @mock.patch('ci_libs.ticket.get_sourcepackage_uri',
                return_value='/api/v1/sourcepackage/5/')
    @mock.patch('ci_libs.ticket.post',
                return_value='http://www.example.com/api/v1/spu/38/')
    def test_create_spu(self, mock_post, mock_sp_uri):
        new_subticket = SubTicket(owner='foobar@example.com', ticket_id=3)
        new_subticket._parse_changes(self.changes)
        new_subticket._create_spu()

    @mock.patch('ci_libs.ticket.post',
                return_value='http://www.example.com/api/v1/artifact/38/')
    def test_create_artifact(self, mock_post):
        new_subticket = SubTicket(owner='foobar@example.com', ticket_id=2)
        file = 'foobar_source.changes'
        location = 'http://www.example.com/path/to/foobar_source.changes'
        new_subticket._create_artifact(file, location)
        mock_post.assert_called_once()

    @mock.patch('ci_libs.ticket.post',
                return_value='http://www.example.com/api/v1/subticket/1/')
    def test_create_subticket(self, mock_post):
        new_subticket = SubTicket(owner='foobar@example.com', ticket_id=1)
        new_subticket._create_subticket()


class TicketTestCase(TestCase):

    def setUp(self):
        self.cli = imp.load_source(
            "ubuntu-ci",
            os.path.join(os.path.dirname(__file__), "../ubuntu-ci")
        )

    @mock.patch('ci_libs.ticket.post',
                return_value='http://www.example.com/api/v1/ticket/38/')
    def test_create_ticket(self, mock_post):
        changes = os.path.join(os.path.dirname(__file__),
                               'data/foobar_0.1-1_source.changes')
        args = ['create_ticket', '-t', 'New feature', '-b', '1234', '-o',
                'someone@example.com', '-d', 'This is a cool new feature',
                '-r', 'baz', '-s', changes]

        new_ticket = Ticket()
        args = self.cli.parse_arguments(args)
        new_ticket._create_ticket(args)


    @mock.patch('ci_libs.ticket.SubTicket._process')
    @mock.patch('ci_libs.ticket.post',
                return_value='http://www.example.com/api/v1/ticket/38/')
    @mock.patch('ci_libs.ticket.patch')
    def test_add_new_ticket(self, mock_patch, mock_post, mock_process):
        changes = os.path.join(os.path.dirname(__file__),
                               'data/foobar_0.1-1_source.changes')
        args = ['create_ticket', '-t', 'New feature', '-b', '1234', '-o',
                'someone@example.com', '-d', 'This is a cool new feature',
                '-r', 'baz', '-s', changes]

        new_ticket = Ticket()
        args = self.cli.parse_arguments(args)
        new_ticket.add_new_ticket(args)
        self.assertEqual(mock_process.call_count, 1)
        self.assertEqual(mock_post.call_count, 1)
        self.assertEqual(mock_patch.call_count, 1)
        self.assertEqual(len(new_ticket.subtickets), 1)
        self.assertTrue("foobar" in new_ticket.subtickets)


    @mock.patch('ci_libs.ticket.SubTicket._process')
    @mock.patch('ci_libs.ticket.post',
                return_value='http://www.example.com/api/v1/ticket/39/')
    @mock.patch('ci_libs.ticket.patch')
    def test_add_new_ticket_multiple_subtickets(self, mock_patch, mock_post,
                                                mock_process):
        changes_1 = os.path.join(os.path.dirname(__file__),
                               'data/foobar_0.1-1_source.changes')
        changes_2 = os.path.join(os.path.dirname(__file__),
                               'data/barfoo_0.1-1_source.changes')

        args = ['create_ticket', '-t', 'New feature', '-b', '4321', '-o',
                'someone@example.com', '-d', 'This is a cool new feature',
                '-r', 'baz', '-s', changes_1, '-s', changes_2]

        new_ticket = Ticket()
        args = self.cli.parse_arguments(args)
        new_ticket.add_new_ticket(args)
        self.assertEqual(mock_process.call_count, 2)
        self.assertEqual(mock_post.call_count, 1)
        self.assertEqual(mock_patch.call_count, 1)

        self.assertEqual(len(new_ticket.subtickets), 2)
        self.assertTrue("foobar" in new_ticket.subtickets)
        self.assertTrue("barfoo" in new_ticket.subtickets)
