#!/usr/bin/python

import os
import sys
import subprocess
import atexit
import shutil
import tempfile
from distutils import spawn
from itertools import chain

# the juju-deployer/ directory.
deployer_dir = os.path.dirname(sys.argv[0])

def check_environment():
    if not os.environ.get('OS_USERNAME'):
        print >>sys.stderr, "Please source a novarc file."
        sys.exit(1)

def check_dependencies():
    '''Exit the application if we are missing any dependency programs'''

    needed_commands = ['juju-deployer', 'cheetah']
    for cmd in needed_commands:
        if not spawn.find_executable(cmd):
            err = "%s is missing. Please install it before continuing." % cmd
            print >>sys.stderr, err
            sys.exit(1)

def populate_templates(deployer_dir, working_dir, specific=None):
    # The command to populate the template.
    cmd_base = ['cheetah', 'fill', '--env', '-p']

    for tmpl in os.listdir(deployer_dir):
        if not tmpl.endswith('.yaml.tmpl'):
            continue
        if specific and tmpl[:-10] not in specific:
                continue
        # Strip the .tmpl off .yaml.tmpl and write a novarc-populated version
        # of the yaml to our working directory.
        with open(os.path.join(working_dir, tmpl[:-5]), 'w') as out:
            args = [os.path.join(deployer_dir, tmpl)]
            subprocess.check_call(cmd_base + args, stdout=out)

def generate_argument_list(d, specific=None):
    for f in os.listdir(d):
        if not f.endswith('.yaml'):
            continue
        # If we're looking for a specific set of components to deploy and this
        # isn't one of them, continue.
        if specific and f[:-5] not in specific:
            continue
        yield ('-c', os.path.join(d, f))


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    # We only want to deploy some specific deployer configs.
    specific = args

    check_environment()
    check_dependencies()

    # A place to put our generated .yaml files.
    working_dir = tempfile.mkdtemp()
    atexit.register(lambda: shutil.rmtree(working_dir))

    # Create .yaml files from .yaml.tmpl files.
    populate_templates(deployer_dir, working_dir, specific)

    # Generate an argument list of -c FILE for each .yaml file in our working
    # directory.
    i = generate_argument_list(working_dir, specific)
    generated = list(chain.from_iterable(i))

    # Generate an argument list of -c FILE for each .yaml file in
    # 'deployer_dir' (juju-deployer/).
    i = generate_argument_list(deployer_dir, specific)
    static = list(chain.from_iterable(i))

    args = ['juju-deployer'] + generated + static + ['ci-airline-staging']
    print 'Running %s' % ' '.join(args)
    subprocess.check_call(args)

if __name__ == '__main__':
    main()
