#!/usr/bin/env python

import json
import os
import unittest

import httplib2
import yaml

import amulet


opd = os.path.dirname

root_dir = opd(opd(opd((os.path.abspath(__file__)))))


class AmuletDeployment(object):
    """Fixture for a deployed set of juju services from a deployer config"""

    def __init__(self, deployer_cfg, juju_env=None, sentries=False):
        self.deployment = amulet.Deployment(juju_env, sentries=sentries)
        with open(deployer_cfg) as f:
            jd_script = yaml.safe_load(f.read())
        # Use juju_env given or defaulted above rather than taking the name
        # from the deployer config and assuming an env of that name exists
        script = {self.deployment.juju_env: jd_script.values()[0]}
        self.deployment.load(script)

    def setUp(self):
        self.deployment.setup()


class TestTestRunner(unittest.TestCase):
    """Integration tests for test runner service run on a juju deployment"""

    def setUp(self):
        super(TestTestRunner, self).setUp()
        self.deployer = AmuletDeployment(os.path.join(root_dir,
                                                      'juju-deployer',
                                                      'test-runner.yaml'))
        self.deployer.setUp()
        self.status = amulet.waiter.status(self.deployer.deployment.juju_env)

    def restish_host_addr(self):
        units = self.status['services']['tr-restish']['units']
        restish_unit = units['tr-restish/0']
        ip = restish_unit['public-address']
        port, _ = restish_unit['open-ports'][0].split('/')
        return ip, port

    def post_test_image(self, test_request_id, image_url, package_name):
        url = 'http://{}:{}/api/v1/test_image'.format(
            *self.restish_host_addr())
        client = httplib2.Http()
        resp, content = client.request(
            url, 'POST',
            headers={'Content-Type': 'application/json'},
            body=json.dumps({'test_request_id': test_request_id,
                             'image_url': image_url,
                             'package_name': package_name,
                             }),
            )
        return resp, content

    def test_test_image_succeeds(self):
        resp, content = self.post_test_image('t-rex', 'http://foo/image',
                                             'uci-tests')
        self.assertEqual('200', resp['status'])
        self.assertEqual('', content)


if __name__ == "__main__":
    unittest.main()
