# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from model_mommy import mommy
from ci_utils.tastypie.test import TastypieTestCase
from ticket.models import (get_enum_title, SubTicketWorkflowStep,
                           SubTicketWorkflowStepStatus, TicketWorkflowStep,
                           TicketWorkflowStepStatus)


class TicketFullReadAPITest(TastypieTestCase):

    def setUp(self):
        super(TicketFullReadAPITest, self).setUp('/api/v1')
        self.ticket = mommy.make('Ticket')
        self.sourcepackage = mommy.make('SourcePackage')
        self.spu = mommy.make('SourcePackageUpload',
                              sourcepackage=self.sourcepackage)
        self.subticket = mommy.make('SubTicket', ticket=self.ticket,
                                    source_package_upload=self.spu)
        self.artifact = mommy.make('Artifact', subticket=self.subticket)
        self.maxDiff = None

    def test_get_fullticket_api(self):
        obj = self.getResource('fullticket/')
        self.assertEqual(obj['objects'][0], {
            u'status': get_enum_title(self.ticket.status,
                                      TicketWorkflowStepStatus),
            u'current_workflow_step': get_enum_title(
                self.ticket.current_workflow_step, TicketWorkflowStep),
            u'description': unicode(self.ticket.description),
            u'title': unicode(self.ticket.title),
            u'subticket': [{
                u'status': get_enum_title(self.subticket.status,
                                          SubTicketWorkflowStepStatus),
                u'current_workflow_step': get_enum_title(
                    self.subticket.current_workflow_step,
                    SubTicketWorkflowStep),
                u'artifact': [{
                    u'resource_uri': u'/api/v1/fullartifact/{0}/'.format(
                        self.artifact.pk),
                    u'type': unicode(self.artifact.type),
                    u'id': self.artifact.pk,
                    u'reference': unicode(self.artifact.reference),
                    u'name': unicode(self.artifact.name)}],
                u'assignee': unicode(self.subticket.assignee),
                u'source_package_upload': {
                    u'version': unicode(self.spu.version),
                    u'sourcepackage': {
                        u'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                            self.sourcepackage.pk),
                        u'id': self.sourcepackage.pk,
                        u'name': unicode(self.sourcepackage.name)},
                    u'id': self.spu.pk,
                    u'resource_uri':
                    u'/api/v1/spu/{0}/'.format(self.spu.pk)},
                u'id': self.subticket.pk,
                u'resource_uri': u'/api/v1/fullsubticket/{0}/'.format(
                    self.subticket.pk)}],
            u'bug_id': self.ticket.bug_id,
            u'created': unicode(self.ticket.created.strftime(
                "%Y-%m-%dT%H:%M:%S.%f")),
            u'owner': unicode(self.ticket.owner),
            u'base_image': unicode(self.ticket.base_image),
            u'id': self.ticket.pk,
            u'resource_uri': u'/api/v1/fullticket/{0}/'.format(self.ticket.pk),
            u'updated': unicode(self.ticket.updated.strftime(
                "%Y-%m-%dT%H:%M:%S.%f")),
            u'added_binaries': self.ticket.added_binaries,
            u'removed_binaries': self.ticket.removed_binaries,
        })

    def test_get_fullartifact_api(self):
        obj = self.getResource('fullartifact/')
        self.assertEqual(obj['objects'][0], {
            u'resource_uri': u'/api/v1/fullartifact/{0}/'.format(
                self.artifact.pk),
            u'type': unicode(self.artifact.type),
            u'id': self.artifact.pk,
            u'reference': unicode(self.artifact.reference),
            u'name': unicode(self.artifact.name),
        })

    def test_get_fullsubticket_api(self):
        obj = self.getResource('fullsubticket/')
        self.assertEqual(obj['objects'][0], {
            u'status': get_enum_title(self.subticket.status,
                                      SubTicketWorkflowStepStatus),
            u'current_workflow_step': get_enum_title(
                self.subticket.current_workflow_step,
                SubTicketWorkflowStep),
            u'artifact': [{
                u'resource_uri': u'/api/v1/fullartifact/{0}/'.format(
                    self.artifact.pk),
                u'type': unicode(self.artifact.type),
                u'id': self.artifact.pk,
                u'reference': unicode(self.artifact.reference),
                u'name': unicode(self.artifact.name)}],
            u'assignee': unicode(self.subticket.assignee),
            u'source_package_upload': {
                u'version': unicode(self.spu.version),
                u'sourcepackage': {
                    u'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                        self.sourcepackage.pk),
                    u'id': self.sourcepackage.pk,
                    u'name': unicode(self.sourcepackage.name)},
                u'id': self.spu.pk,
                u'resource_uri':
                u'/api/v1/spu/{0}/'.format(self.spu.pk)},
            u'id': self.subticket.pk,
            u'resource_uri': u'/api/v1/fullsubticket/{0}/'.format(
                self.subticket.pk),
        })


class GetNextTicketTest(TastypieTestCase):
    """
    Test that /next/ returns the current inprogress ticket, or the next
    'queued' ticket.
    """
    def setUp(self):
        super(GetNextTicketTest, self).setUp('/api/v1')
        self.ticket_1 = mommy.make(
            'Ticket', current_workflow_step=int(TicketWorkflowStep.NEW))
        self.ticket_2 = mommy.make(
            'Ticket', current_workflow_step=int(TicketWorkflowStep.QUEUED))
        self.ticket_3 = mommy.make(
            'Ticket', current_workflow_step=int(TicketWorkflowStep.QUEUED))
        self.ticket_4 = mommy.make(
            'Ticket', current_workflow_step=int(TicketWorkflowStep.COMPLETED))
        self.sourcepackage = mommy.make('SourcePackage')
        self.spu = mommy.make('SourcePackageUpload',
                              sourcepackage=self.sourcepackage)
        self.subticket_1 = mommy.make('SubTicket', ticket=self.ticket_1,
                                      source_package_upload=self.spu)
        self.artifact_1 = mommy.make('Artifact', subticket=self.subticket_1)
        self.subticket_2 = mommy.make('SubTicket', ticket=self.ticket_2,
                                      source_package_upload=self.spu)
        self.artifact_2 = mommy.make('Artifact', subticket=self.subticket_2)
        self.subticket_3 = mommy.make('SubTicket', ticket=self.ticket_3,
                                      source_package_upload=self.spu)
        self.artifact_3 = mommy.make('Artifact', subticket=self.subticket_3)
        self.subticket_4 = mommy.make('SubTicket', ticket=self.ticket_4,
                                      source_package_upload=self.spu)
        self.artifact_4 = mommy.make('Artifact', subticket=self.subticket_4)

    def test_queued_ticket_next(self):
        obj = self.getResource('next/')
        self.assertEqual(len(obj['objects']), 1)
        self.assertEqual(obj['objects'][0][u'resource_uri'],
                         u'/api/v1/next/{0}/'.format(
                             self.ticket_2.pk))
        self.assertEqual(
            obj['objects'][0][u'current_workflow_step'], get_enum_title(
                self.ticket_2.current_workflow_step, TicketWorkflowStep))

    def test_pkg_building_ticket_next(self):
        self.ticket_3.current_workflow_step = int(
            TicketWorkflowStep.PKG_BUILDING)
        self.ticket_3.save()
        obj = self.getResource('next/')
        self.assertEqual(len(obj['objects']), 1)
        self.assertEqual(obj['objects'][0][u'resource_uri'],
                         u'/api/v1/next/{0}/'.format(
                             self.ticket_3.pk))
        self.assertEqual(
            obj['objects'][0][u'current_workflow_step'], get_enum_title(
                self.ticket_3.current_workflow_step, TicketWorkflowStep))

    def test_image_building_ticket_next(self):
        self.ticket_3.current_workflow_step = int(
            TicketWorkflowStep.IMAGE_BUILDING)
        self.ticket_3.save()
        obj = self.getResource('next/')
        self.assertEqual(len(obj['objects']), 1)
        self.assertEqual(obj['objects'][0][u'resource_uri'],
                         u'/api/v1/next/{0}/'.format(
                             self.ticket_3.pk))
        self.assertEqual(
            obj['objects'][0][u'current_workflow_step'], get_enum_title(
                self.ticket_3.current_workflow_step, TicketWorkflowStep))

    def test_image_testing_ticket_next(self):
        self.ticket_3.current_workflow_step = int(
            TicketWorkflowStep.IMAGE_TESTING)
        self.ticket_3.save()
        obj = self.getResource('next/')
        self.assertEqual(len(obj['objects']), 1)
        self.assertEqual(obj['objects'][0][u'resource_uri'],
                         u'/api/v1/next/{0}/'.format(
                             self.ticket_3.pk))
        self.assertEqual(
            obj['objects'][0][u'current_workflow_step'], get_enum_title(
                self.ticket_3.current_workflow_step, TicketWorkflowStep))

    def test_pkg_publishing_ticket_next(self):
        self.ticket_3.current_workflow_step = int(
            TicketWorkflowStep.PKG_PUBLISHING)
        self.ticket_3.save()
        obj = self.getResource('next/')
        self.assertEqual(len(obj['objects']), 1)
        self.assertEqual(obj['objects'][0][u'resource_uri'],
                         u'/api/v1/next/{0}/'.format(
                             self.ticket_3.pk))
        self.assertEqual(
            obj['objects'][0][u'current_workflow_step'], get_enum_title(
                self.ticket_3.current_workflow_step, TicketWorkflowStep))

    def test_ticket_complete_next_new_ticket_next(self):
        self.ticket_3.current_workflow_step = int(
            TicketWorkflowStep.COMPLETED)
        self.ticket_3.save()
        obj = self.getResource('next/')
        self.assertEqual(len(obj['objects']), 1)
        self.assertEqual(obj['objects'][0][u'resource_uri'],
                         u'/api/v1/next/{0}/'.format(
                             self.ticket_2.pk))
        self.assertEqual(
            obj['objects'][0][u'current_workflow_step'], get_enum_title(
                self.ticket_2.current_workflow_step, TicketWorkflowStep))

    def test_no_queued_or_in_progress_ticket_next(self):
        self.ticket_2.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_2.save()
        self.ticket_3.current_workflow_step = int(
            TicketWorkflowStep.NEW)
        self.ticket_3.save()
        obj = self.getResource('next/')
        self.assertEqual(len(obj['objects']), 0)
