YUI.add('webui', function(Y) {
  Y.webui = {
    url_prefix: "/api/",
    get_config: function() {
      Y.io("config.json", {
        on: {
          success: function(tx, r) {
            var data;

            try {
              data = Y.JSON.parse(r.responseText);
            }
            catch (e) {
              alert("JSON Parse failed");
              return;
            }
          }
        }
      });
    },
    update_tickets: function() {
      if (Y.webui.main_table) {
        Y.webui.get_tickets();
      }

      var running = Y.all(".running");
      if (running.size() > 0) {
        refresh_timeout = setTimeout(update_tickets, interval);
      }
    },
    get_ticket: function(ticket_id, elem) {
      var table = null;
      data = Y.io(Y.webui.url_prefix + "fullticket/" + ticket_id + "/", {
        on: {
          success: function(tx, r) {
            console.log("success");
            var data;

            try {
              data = Y.JSON.parse(r.responseText);
            }
            catch (e) {
              alert("JSON Parse failed");
              return;
            }

            var new_div =  Y.webui.ticket_detail_element(data);
            elem.setHTML(new_div);

          },
          failure: function(tx, r, e) {
            console.log("failure: ", tx, r, e);
          }
        }
      });
    },
    get_tickets: function() {
      container = Y.one('#container');
      data = Y.io(Y.webui.url_prefix + "ticket/", {
        on: {
          success: function(tx, r) {
            console.log("success");
            var data;

            try {
              data = Y.JSON.parse(r.responseText);
            }
            catch (e) {
              alert("JSON Parse failed");
              return;
            }

            for (var i = 0; i < data.objects.length; i++) {
              Y.webui.add_item(data.objects[i]);
            }
          },
          failure: function(tx, r, e) {
            console.log("failure: ", tx, r, e);
          }
        }
      });
    },
    add_subticket: function(data) {
      var main_tr = Y.Node.create("<tr>");
      main_tr.appendChild(Y.Node.create('<th>subticket</th>'));
      for (var key in data) {
        var value = data[key];

        var tr = main_tr.appendChild(Y.Node.create('<tr>'));
        tr.addClass('subticket');
        tr.appendChild(Y.Node.create('<th>' + key + '</th>'));
        tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
      }

      main_tr.addClass('subticket');
      return main_tr;
    },
    make_subticket: function(data) {
      var div = Y.Node.create("<div class='subticket'></div>");

      var table = div.appendChild(Y.Node.create("<table>"));
      var hidden_fields = Y.Array(['id', 'resource_uri']);

      for (var key in data) {
        // Skip hidden fields
        if (Y.Array.indexOf(hidden_fields, key) != -1) {
          continue;
        }

        var value = data[key];
        if (key == 'artifact') {
          var span = Y.webui.add_artifacts(data['artifact']);
          value = span.getHTML();
        }

        if (key == 'source_package_upload') {
          var span = Y.webui.sp_upload(data['source_package_upload']);
          value = span.getHTML();
        }

        var tr = table.appendChild(Y.Node.create('<tr>'));
        tr.appendChild(Y.Node.create('<th>' + key + ': </th>'));
        tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
      }

      return div;
    },
    basename: function(path) {
      return path.replace(/^.*\//, "");
    },
    add_artifacts: function(data) {
      var span = Y.Node.create('<span></span>');
      for (var i = 0; i < data.length; i++) {
        span.appendChild(Y.webui.artifact_link(data[i]));
        span.appendChild(Y.Node.create("<br/>"));
      }

      return span;
    },
    artifact_link: function(data) {
      var name = Y.webui.basename(data['name']);
      var link = Y.Node.create("<a/>");
      link.set('href', data['reference']);
      link.set('text', name);

      return link;
    },
    sp_upload: function(data) {
      var span = Y.Node.create("<span></span>");

      if (data.sourcepackage) {
        span.setHTML(data.sourcepackage.name + " " + data.version);
      }

      return span;
    },
    ticket_detail_element: function(data) {
      var hidden_fields = Y.Array(['id', 'title', 'subticket', 'resource_uri']);

      div = Y.Node.create("<div id='table_wrapper'/>");
      ticket_div = div.appendChild(Y.Node.create("<div class='ticket'></div>"));
      table = ticket_div.appendChild(Y.Node.create("<table></table>"));

      if (data['title']) {
        div.prepend(Y.Node.create('<h2>' + data['title'] + '</h2>'));
      }

      for (var key in data) {
        var value = data[key];

        if (Y.Array.indexOf(hidden_fields, key) != -1) {
          continue;
        }

        if (key == "title" || key == "subticket") {
          continue;
        }

        tr = Y.Node.create('<tr></tr>');
        table.appendChild(tr);
        tr.appendChild(Y.Node.create('<th>' + key + '</th>'));
        tr.appendChild(Y.Node.create('<td>' + value + '</td>'));
      }

      if (data['subticket']) {
        div.appendChild("<h3>Subtickets</h3>");
        var subticket_count = data['subticket'].length;
        if (subticket_count > 0) {
          for (var i = 0; i < subticket_count; i++) {
            var subticket = data['subticket'][i];

            var subt = Y.webui.make_subticket(subticket);

            div.appendChild(subt);
          }
        }
      }
      return div;
    },
    check_ticket: function(e) {
      var elem = this;
      var id = elem.get('id').replace('item-', '');

      data = Y.io(Y.webui.url_prefix +  "status/" + id + "/", {
        on: {
          success: function(tx, r) {
            var data;

            try {
              data = Y.JSON.parse(r.responseText);
            }
            catch (e) {
              alert("JSON Parse failed");
              return;
            }

            var update_fields = ['status', 'title', 'current_workflow_step', 'updated'];

            for (i = 0; i < update_fields.length; i++) {
              var field_name = update_fields[i];
              var field = elem.one('#' + field_name + '-' + id);
              var old_value = field.getHTML();
              var new_value = data[field_name];
              if (field && new_value && old_value != new_value) {
                field.setHTML(new_value);
              }
            }
            // HACK for development testing
            var new_status = data['status'];
            if (new_status == 440) {
              elem.addClass('red');
            } else {
              elem.removeClass('red');
            }
          }
        }
      });
    },
    add_item: function(item) {
      var entry = item;
      var id = entry['id'];
      var table = Y.webui.main_table;

      if (table) {
        var tr = table.addRow({
          title: entry['title'],
          current_workflow_step: entry['current_workflow_step'],
          status: entry['status'],
          ticket_id: id
        });
      }
    },
    make_main_table: function(div_id) {

      var columns = [
        {key: 'title', label: 'Title'},
        {key: 'current_workflow_step', label: 'Workflow Step'},
        {key: 'status', label: 'Status'}
      ]
      var data = [];

      var main_table = new Y.DataTable({
        columnset: columns,
        recordset: data
      });
      main_table.render(div_id);

      Y.webui.main_table = main_table;
    }
  };
}, '0.0.1', {requires: ['io', 'node', 'json-parse', 'datatable']});
